@@ -3,12 +3,13 @@
 See https://argoproj.github.io/argo-workflows/workflow-concepts/#container
 for more on containers.
 """
+
 from __future__ import annotations
 
 from typing import List, Optional
 
+from hera.workflows._meta_mixins import CallableTemplateMixin
 from hera.workflows._mixins import (
-    CallableTemplateMixin,
     ContainerMixin,
     EnvIOMixin,
     ResourceMixin,
@@ -46,6 +47,7 @@ class Container(
 
     def _build_container(self) -> _ModelContainer:
         """Builds the generated `Container` representation."""
+        assert self.image
         return _ModelContainer(
             args=self.args,
             command=self.command,
@@ -57,7 +59,9 @@ class Container(
             liveness_probe=self.liveness_probe,
             ports=self.ports,
             readiness_probe=self.readiness_probe,
+            resize_policy=self.resize_policy,
             resources=self._build_resources(),
+            restart_policy=self.restart_policy,
             security_context=self.security_context,
             startup_probe=self.startup_probe,
             stdin=self.stdin,
@@ -82,7 +86,7 @@ class Container(
             executor=self.executor,
             fail_fast=self.fail_fast,
             host_aliases=self.host_aliases,
-            init_containers=self.init_containers,
+            init_containers=self._build_init_containers(),
             inputs=self._build_inputs(),
             memoize=self.memoize,
             metadata=self._build_metadata(),
