@@ -1,9 +1,11 @@
 """Data module provides Hera objects for Argo data functionality, such as sourcing data + applying transformations."""
+
 from typing import List, Union
 
 from hera.expr._node import Node
 from hera.workflows import models as m
-from hera.workflows._mixins import CallableTemplateMixin, IOMixin, TemplateMixin
+from hera.workflows._meta_mixins import CallableTemplateMixin
+from hera.workflows._mixins import IOMixin, TemplateMixin
 from hera.workflows.artifact import Artifact
 
 
@@ -44,7 +46,7 @@ class Data(TemplateMixin, IOMixin, CallableTemplateMixin):
             executor=self.executor,
             fail_fast=self.fail_fast,
             host_aliases=self.host_aliases,
-            init_containers=self.init_containers,
+            init_containers=self._build_init_containers(),
             inputs=self._build_inputs(),
             outputs=self._build_outputs(),
             memoize=self.memoize,
