@@ -1,12 +1,14 @@
 """The HTTP template module provides functionality and objects required for executing HTTP calls in workflows."""
+
 from typing import List, Optional
 
-from hera.workflows._mixins import CallableTemplateMixin, IOMixin, TemplateMixin
+from hera.workflows._meta_mixins import CallableTemplateMixin
+from hera.workflows._mixins import IOMixin, TemplateMixin
 from hera.workflows.models import (
     HTTP as _ModelHTTP,
     HTTPBodySource,
     Template as _ModelTemplate,
-    V1HTTPHeader as HTTPHeader,
+    V1alpha1HTTPHeader as HTTPHeader,
 )
 
 
@@ -46,7 +48,7 @@ class HTTP(TemplateMixin, IOMixin, CallableTemplateMixin):
             fail_fast=self.fail_fast,
             host_aliases=self.host_aliases,
             http=self._build_http_template(),
-            init_containers=self.init_containers,
+            init_containers=self._build_init_containers(),
             memoize=self.memoize,
             metadata=self._build_metadata(),
             inputs=self._build_inputs(),
