@@ -4,16 +4,23 @@ See https://argoproj.github.io/argo-workflows/workflow-concepts/#script
 for more on scripts.
 """
 
+import ast
 import copy
 import inspect
+import sys
 import textwrap
 from abc import abstractmethod
 from functools import wraps
 from typing import (
     Any,
     Callable,
+    Dict,
+    Generic,
     List,
+    Literal,
     Optional,
+    Protocol,
+    Sequence,
     Tuple,
     Type,
     TypeVar,
@@ -22,44 +29,77 @@ from typing import (
     overload,
 )
 
-from pydantic import root_validator, validator
+if sys.version_info >= (3, 10):
+    from types import NoneType
+else:
+    NoneType = type(None)
+
+
 from typing_extensions import ParamSpec, get_args, get_origin
 
 from hera.expr import g
 from hera.shared import BaseMixin, global_config
+from hera.shared._global_config import (
+    _SCRIPT_PYDANTIC_IO_FLAG,
+    _flag_enabled,
+)
+from hera.shared._pydantic import _PYDANTIC_VERSION, root_validator, validator
+from hera.shared._type_util import (
+    construct_io_from_annotation,
+    get_workflow_annotation,
+    is_subscripted,
+    origin_type_issupertype,
+)
+from hera.shared.serialization import serialize
 from hera.workflows._context import _context
+from hera.workflows._meta_mixins import CallableTemplateMixin
 from hera.workflows._mixins import (
-    CallableTemplateMixin,
+    ArgumentsT,
     ContainerMixin,
     EnvIOMixin,
+    OneOrMany,
     ResourceMixin,
     TemplateMixin,
     VolumeMountMixin,
 )
-from hera.workflows._unparse import roundtrip
 from hera.workflows.artifact import (
     Artifact,
 )
+from hera.workflows.io.v1 import (
+    Input as InputV1,
+    Output as OutputV1,
+)
+
+try:
+    from hera.workflows.io.v2 import (  # type: ignore
+        Input as InputV2,
+        Output as OutputV2,
+    )
+except ImportError:
+    from hera.workflows.io.v1 import (  # type: ignore
+        Input as InputV2,
+        Output as OutputV2,
+    )
 from hera.workflows.models import (
+    ContinueOn,
     EnvVar,
     Inputs as ModelInputs,
     Lifecycle,
+    LifecycleHook,
     Outputs as ModelOutputs,
     ScriptTemplate as _ModelScriptTemplate,
     SecurityContext,
+    Sequence as _ModelSequence,
     Template as _ModelTemplate,
+    TemplateRef,
     ValueFrom,
 )
-from hera.workflows.parameter import MISSING, Parameter
+from hera.workflows.parameter import Parameter
+from hera.workflows.protocol import Templatable
 from hera.workflows.steps import Step
 from hera.workflows.task import Task
 from hera.workflows.volume import _BaseVolume
 
-try:
-    from typing import Annotated  # type: ignore
-except ImportError:
-    from typing_extensions import Annotated  # type: ignore
-
 
 class ScriptConstructor(BaseMixin):
     """A ScriptConstructor is responsible for generating the source code for a Script given a python callable.
@@ -100,7 +140,7 @@ class Script(
 ):
     """A Script acts as a wrapper around a container.
 
-    In Hera this defaults to a "python:3.8" image specified by global_config.image, which runs a python source
+    In Hera this defaults to a "python:3.9" image specified by global_config.image, which runs a python source
     specified by `Script.source`.
     """
 
@@ -163,7 +203,7 @@ class Script(
                 executor=self.executor,
                 fail_fast=self.fail_fast,
                 host_aliases=self.host_aliases,
-                init_containers=self.init_containers,
+                init_containers=self._build_init_containers(),
                 inputs=self._build_inputs(),
                 memoize=self.memoize,
                 metadata=self._build_metadata(),
@@ -191,7 +231,6 @@ class Script(
 
     def _build_script(self) -> _ModelScriptTemplate:
         assert isinstance(self.constructor, ScriptConstructor)
-        image_pull_policy = self._build_image_pull_policy()
         if _output_annotations_used(cast(Callable, self.source)) and isinstance(
             self.constructor, RunnerScriptConstructor
         ):
@@ -201,7 +240,7 @@ class Script(
                 if self.constructor.volume_for_outputs.mount_path is None:
                     self.constructor.volume_for_outputs.mount_path = self.constructor.outputs_directory
                 self._create_hera_outputs_volume(self.constructor.volume_for_outputs)
-
+        assert self.image
         return self.constructor.transform_script_template_post_build(
             self,
             _ModelScriptTemplate(
@@ -211,13 +250,15 @@ class Script(
                 env_from=self._build_env_from(),
                 image=self.image,
                 # `image_pull_policy` in script wants a string not an `ImagePullPolicy` object
-                image_pull_policy=None if image_pull_policy is None else image_pull_policy.value,
+                image_pull_policy=self._build_image_pull_policy(),
                 lifecycle=self.lifecycle,
                 liveness_probe=self.liveness_probe,
                 name=self.container_name,
                 ports=self.ports,
                 readiness_probe=self.readiness_probe,
+                resize_policy=self.resize_policy,
                 resources=self._build_resources(),
+                restart_policy=self.restart_policy,
                 security_context=self.security_context,
                 source=self.constructor.generate_source(self),
                 startup_probe=self.startup_probe,
@@ -237,10 +278,7 @@ class Script(
         func_parameters: List[Parameter] = []
         func_artifacts: List[Artifact] = []
         if callable(self.source):
-            if global_config.experimental_features["script_annotations"]:
-                func_parameters, func_artifacts = _get_inputs_from_callable(self.source)
-            else:
-                func_parameters = _get_parameters_from_callable(self.source)
+            func_parameters, func_artifacts = _get_inputs_from_callable(self.source)
 
         return cast(Optional[ModelInputs], self._aggregate_callable_io(inputs, func_parameters, func_artifacts, False))
 
@@ -250,9 +288,6 @@ class Script(
         if not callable(self.source):
             return outputs
 
-        if not global_config.experimental_features["script_annotations"]:
-            return outputs
-
         outputs_directory = None
         if isinstance(self.constructor, RunnerScriptConstructor):
             outputs_directory = self.constructor.outputs_directory or self.constructor.DEFAULT_HERA_OUTPUTS_DIRECTORY
@@ -312,32 +347,17 @@ class Script(
         """Add given volume to the script template for the automatic saving of the hera outputs."""
         assert isinstance(self.constructor, RunnerScriptConstructor)
 
-        if not isinstance(self.volumes, list) and self.volumes is not None:
-            self.volumes = [self.volumes]
-        elif self.volumes is None:
+        if self.volumes is None:
             self.volumes = []
+        elif isinstance(self.volumes, Sequence):
+            self.volumes = list(self.volumes)
+        elif not isinstance(self.volumes, list):
+            self.volumes = [self.volumes]
 
         if volume not in self.volumes:
             self.volumes.append(volume)
 
 
-def _get_parameters_from_callable(source: Callable) -> List[Parameter]:
-    # If there are any kwargs arguments associated with the function signature,
-    # we store these as we can set them as default values for argo arguments
-    parameters = []
-
-    for p in inspect.signature(source).parameters.values():
-        if p.default != inspect.Parameter.empty and p.kind == inspect.Parameter.POSITIONAL_OR_KEYWORD:
-            default = p.default
-        else:
-            default = MISSING
-
-        param = Parameter(name=p.name, default=default)
-        parameters.append(param)
-
-    return parameters
-
-
 def _get_outputs_from_return_annotation(
     source: Callable,
     outputs_directory: Optional[str],
@@ -356,11 +376,31 @@ def _get_outputs_from_return_annotation(
                 annotation.value_from = ValueFrom(path=outputs_directory + f"/parameters/{annotation.name}")
             parameters.append(annotation)
 
-    if get_origin(inspect.signature(source).return_annotation) is Annotated:
-        append_annotation(get_args(inspect.signature(source).return_annotation)[1])
-    elif get_origin(inspect.signature(source).return_annotation) is tuple:
-        for annotation in get_args(inspect.signature(source).return_annotation):
-            append_annotation(get_args(annotation)[1])
+    return_annotation = inspect.signature(source).return_annotation
+    if param_or_artifact := get_workflow_annotation(return_annotation):
+        append_annotation(param_or_artifact)
+    elif get_origin(return_annotation) is tuple:
+        for annotation in get_args(return_annotation):
+            if isinstance(annotation, type) and issubclass(annotation, (OutputV1, OutputV2)):
+                raise ValueError("Output cannot be part of a tuple output")
+
+            if param_or_artifact := get_workflow_annotation(annotation):
+                append_annotation(param_or_artifact)
+    elif isinstance(return_annotation, type) and issubclass(return_annotation, (OutputV1, OutputV2)):
+        if not _flag_enabled(_SCRIPT_PYDANTIC_IO_FLAG):
+            raise ValueError(
+                (
+                    "Unable to instantiate {} since it is an experimental feature."
+                    " Please turn on experimental features by setting "
+                    '`hera.shared.global_config.experimental_features["{}"] = True`.'
+                    " Note that experimental features are unstable and subject to breaking changes."
+                ).format(return_annotation, _SCRIPT_PYDANTIC_IO_FLAG)
+            )
+
+        output_class = return_annotation
+        for output in output_class._get_outputs():
+            append_annotation(output)
+
     return parameters, artifacts
 
 
@@ -373,31 +413,19 @@ def _get_outputs_from_parameter_annotations(
     artifacts: List[Artifact] = []
 
     for name, p in inspect.signature(source).parameters.items():
-        if get_origin(p.annotation) is not Annotated:
-            continue
-        annotation = get_args(p.annotation)[1]
-
-        if not isinstance(annotation, (Artifact, Parameter)):
-            raise ValueError(f"The output {type(annotation)} cannot be used as an annotation.")
-
+        annotation = construct_io_from_annotation(name, p.annotation)
         if not annotation.output:
             continue
 
-        new_object = annotation.copy()
+        if isinstance(annotation, Parameter) and annotation.value_from is None and outputs_directory is not None:
+            annotation.value_from = ValueFrom(path=outputs_directory + f"/parameters/{annotation.name}")
+        elif isinstance(annotation, Artifact) and annotation.path is None and outputs_directory is not None:
+            annotation.path = outputs_directory + f"/artifacts/{annotation.name}"
 
-        # use the function parameter name when not provided by user
-        if not new_object.name:
-            new_object.name = name
-
-        if isinstance(new_object, Parameter) and new_object.value_from is None and outputs_directory is not None:
-            new_object.value_from = ValueFrom(path=outputs_directory + f"/parameters/{new_object.name}")
-        elif isinstance(new_object, Artifact) and new_object.path is None and outputs_directory is not None:
-            new_object.path = outputs_directory + f"/artifacts/{new_object.name}"
-
-        if isinstance(new_object, Artifact):
-            artifacts.append(new_object)
-        elif isinstance(new_object, Parameter):
-            parameters.append(new_object)
+        if isinstance(annotation, Artifact):
+            artifacts.append(annotation)
+        elif isinstance(annotation, Parameter):
+            parameters.append(annotation)
 
     return parameters, artifacts
 
@@ -406,62 +434,95 @@ def _get_inputs_from_callable(source: Callable) -> Tuple[List[Parameter], List[A
     """Return all inputs from the function.
 
     This includes all basic Python function parameters, and all parameters with a Parameter or Artifact annotation.
+    For the Pydantic IO experimental feature, any input parameter which is a subclass of Input, the fields of the
+    class will be used as inputs, rather than the class itself.
+
+    Note, the given Parameter/Artifact names in annotations of different inputs could clash, which will raise a ValueError.
     """
     parameters = []
     artifacts = []
 
     for func_param in inspect.signature(source).parameters.values():
-        if get_origin(func_param.annotation) is not Annotated or not isinstance(
-            get_args(func_param.annotation)[1], (Artifact, Parameter)
-        ):
+        if not is_subscripted(func_param.annotation) and issubclass(func_param.annotation, (InputV1, InputV2)):
+            if not _flag_enabled(_SCRIPT_PYDANTIC_IO_FLAG):
+                raise ValueError(
+                    (
+                        "Unable to instantiate {} since it is an experimental feature."
+                        " Please turn on experimental features by setting "
+                        '`hera.shared.global_config.experimental_features["{}"] = True`.'
+                        " Note that experimental features are unstable and subject to breaking changes."
+                    ).format(func_param.annotation, _SCRIPT_PYDANTIC_IO_FLAG)
+                )
+
+            if len(inspect.signature(source).parameters) != 1:
+                raise SyntaxError("Only one function parameter can be specified when using an Input.")
+
+            input_class = func_param.annotation
             if (
                 func_param.default != inspect.Parameter.empty
                 and func_param.kind == inspect.Parameter.POSITIONAL_OR_KEYWORD
             ):
-                default = func_param.default
+                object_override = func_param.default
+                parameters.extend(input_class._get_parameters(object_override=object_override))
             else:
-                default = MISSING
+                parameters.extend(input_class._get_parameters())
+
+            artifacts.extend(input_class._get_artifacts(add_missing_path=True))
 
-            parameters.append(Parameter(name=func_param.name, default=default))
         else:
-            annotation = get_args(func_param.annotation)[1]
-
-            if annotation.output:
+            io = construct_io_from_annotation(func_param.name, func_param.annotation)
+            if io.output:
                 continue
 
-            # Create a new object so we don't modify the Workflow itself
-            new_object = annotation.copy()
-            if not new_object.name:
-                new_object.name = func_param.name
+            if isinstance(io, Artifact):
+                if io.path is None:
+                    io.path = io._get_default_inputs_path()
 
-            if isinstance(new_object, Artifact):
-                if new_object.path is None:
-                    new_object.path = new_object._get_default_inputs_path()
-
-                artifacts.append(new_object)
-            elif isinstance(new_object, Parameter):
+                artifacts.append(io)
+            elif isinstance(io, Parameter):
+                if io.default is not None:
+                    raise ValueError(
+                        "default cannot be set via the Parameter's default, use a Python default value instead."
+                    )
                 if func_param.default != inspect.Parameter.empty:
-                    if new_object.default is not None:
-                        raise ValueError(
-                            "default cannot be set via both the function parameter default and the Parameter's default"
-                        )
-                    new_object.default = str(func_param.default)
-                parameters.append(new_object)
+                    io.default = serialize(func_param.default)
+
+                if origin_type_issupertype(func_param.annotation, NoneType) and io.default != "null":
+                    raise ValueError(f"Optional parameter '{func_param.name}' must have a default value of None.")
+                parameters.append(io)
 
     return parameters, artifacts
 
 
 def _extract_return_annotation_output(source: Callable) -> List:
     """Extract the output annotations from the return annotation of the function signature."""
-    output = []
+    output: List[Union[Tuple[type, Union[Parameter, Artifact]], Type[Union[OutputV1, OutputV2]]]] = []
 
-    origin_type = get_origin(inspect.signature(source).return_annotation)
-    annotation_args = get_args(inspect.signature(source).return_annotation)
-    if origin_type is Annotated:
+    return_annotation = inspect.signature(source).return_annotation
+    origin_type = get_origin(return_annotation)
+    annotation_args = get_args(return_annotation)
+    if get_workflow_annotation(return_annotation):
         output.append(annotation_args)
     elif origin_type is tuple:
-        for annotated_type in annotation_args:
-            output.append(get_args(annotated_type))
+        workflow_args = [
+            get_args(annotated_type) for annotated_type in annotation_args if get_workflow_annotation(annotated_type)
+        ]
+
+        # If all tuple elements are annotated as Parameter/Artifact
+        if len(workflow_args) == len(annotation_args):
+            output.extend(workflow_args)
+        # Only some tuple elements are annotated as Parameter/Artifact
+        elif workflow_args:
+            raise ValueError(
+                f"Function '{source.__name__}' output has partially annotated tuple return type. "
+                "Tuple elements must be all Annotated as Parameter/Artifact, or contain no Parameter/Artifact annotations for a raw tuple return type."
+            )
+    elif (
+        origin_type is None
+        and isinstance(return_annotation, type)
+        and issubclass(return_annotation, (OutputV1, OutputV2))
+    ):
+        output.append(return_annotation)
 
     return output
 
@@ -474,11 +535,9 @@ def _extract_all_output_annotations(source: Callable) -> List:
     output = []
 
     for _, func_param in inspect.signature(source).parameters.items():
-        if get_origin(func_param.annotation) is Annotated:
-            annotation_args = get_args(func_param.annotation)
-            annotated_type = annotation_args[1]
-            if isinstance(annotated_type, (Artifact, Parameter)) and annotated_type.output:
-                output.append(annotation_args)
+        io = construct_io_from_annotation(func_param.name, func_param.annotation)
+        if io.output:
+            output.append(io)
 
     output.extend(_extract_return_annotation_output(source))
 
@@ -490,8 +549,6 @@ def _output_annotations_used(source: Callable) -> bool:
 
     This includes parameters marked as outputs and the return annotation of `source`.
     """
-    if not global_config.experimental_features["script_annotations"]:
-        return False
     if not callable(source):
         return False
 
@@ -500,41 +557,114 @@ def _output_annotations_used(source: Callable) -> bool:
 
 FuncIns = ParamSpec("FuncIns")  # For input types of given func to script decorator
 FuncR = TypeVar("FuncR")  # For return type of given func to script decorator
-ScriptIns = ParamSpec("ScriptIns")  # For attribute types of Script
+FuncRCov = TypeVar("FuncRCov", covariant=True)
+
+ScriptIns = ParamSpec("ScriptIns")  # For input attributes of Script class
 
 
-def _take_annotation_from(
-    _: Callable[
-        ScriptIns,
-        Callable[[Callable[FuncIns, FuncR]], Union[Callable[FuncIns, FuncR], Callable[ScriptIns, Union[Task, Step]]]],
-    ],
+class _ScriptDecoratedFunction(Generic[FuncIns, FuncRCov], Protocol):
+    """Type assigned to functions decorated with @script."""
+
+    # Note: For more details about overload-overlap, see https://github.com/python/typeshed/issues/12178
+
+    @overload
+    def __call__(  # type: ignore [overload-overlap]
+        self, *args: FuncIns.args, **kwargs: FuncIns.kwargs
+    ) -> FuncRCov:
+        # Note: this overload is for calling the decorated function.
+        # No docstring is provided, so VS Code will use the docstring of the decorated function.
+        ...
+
+    @overload
+    def __call__(  # type: ignore [overload-overlap, misc]
+        self,
+    ) -> Optional[Union[Step, Task]]:
+        """Create a Step or Task or add the script as a template to the workflow, depending on the context.
+
+        * Under a DAG context, creates and returns a Task.
+        * Under a Steps or Parallel context, creates and returns a Step.
+        * Under a Workflow context, adds the script as a template to the Workflow and returns None.
+
+        Use `assert isinstance(result, Step)` or `assert isinstance(result, Task)` to select
+        the correct type if using a type-checker.
+        """
+
+    @overload
+    def __call__(  # type: ignore [overload-overlap]
+        self,
+        *,
+        name: str = ...,
+        continue_on: Optional[ContinueOn] = ...,
+        hooks: Optional[Dict[str, LifecycleHook]] = ...,
+        on_exit: Optional[Union[str, Templatable]] = ...,
+        template: Optional[Union[str, _ModelTemplate, TemplateMixin, CallableTemplateMixin]] = ...,
+        template_ref: Optional[TemplateRef] = ...,
+        inline: Optional[Union[_ModelTemplate, TemplateMixin]] = ...,
+        when: Optional[str] = ...,
+        with_sequence: Optional[_ModelSequence] = ...,
+        arguments: ArgumentsT = ...,
+        with_param: Optional[Any] = ...,
+        with_items: Optional[OneOrMany[Any]] = ...,
+    ) -> Union[Step, Task]:
+        """Create a Step or Task, depending on context.
+
+        * Under a DAG context, creates and returns a Task.
+        * Under a Steps or Parallel context, creates and returns a Step.
+
+        Use `assert isinstance(result, Step)` or `assert isinstance(result, Task)` to select
+        the correct type if using a type-checker.
+        """
+        # Note: signature must match the Step constructor, except that while name is required for Step,
+        # it is automatically inferred from the name of the decorated function when invoked.
+
+    @overload
+    def __call__(  # type: ignore [overload-overlap]
+        self,
+        *,
+        name: str = ...,
+        continue_on: Optional[ContinueOn] = ...,
+        hooks: Optional[Dict[str, LifecycleHook]] = ...,
+        on_exit: Optional[Union[str, Templatable]] = ...,
+        template: Optional[Union[str, _ModelTemplate, TemplateMixin, CallableTemplateMixin]] = ...,
+        template_ref: Optional[TemplateRef] = ...,
+        inline: Optional[Union[_ModelTemplate, TemplateMixin]] = ...,
+        when: Optional[str] = ...,
+        with_sequence: Optional[_ModelSequence] = ...,
+        arguments: ArgumentsT = ...,
+        with_param: Optional[Any] = ...,
+        with_items: Optional[OneOrMany[Any]] = ...,
+        dependencies: Optional[List[str]] = ...,
+        depends: Optional[str] = ...,
+    ) -> Task:
+        """Create and return a Task.
+
+        Must be invoked under a DAG context.
+        """
+        # Note: signature must match the Task constructor, except that while name is required for Task,
+        # it is automatically inferred from the name of the decorated function when invoked.
+
+
+# Pass actual class of Script to bind inputs to the ParamSpec above
+def _add_type_hints(
+    _script: Callable[ScriptIns, Script],
 ) -> Callable[
-    [Callable],
+    ...,
     Callable[
-        ScriptIns,
-        Callable[[Callable[FuncIns, FuncR]], Union[Callable[FuncIns, FuncR], Callable[ScriptIns, Union[Task, Step]]]],
+        ScriptIns,  # this adds Script type hints to the underlying *library* function kwargs, i.e. `script`
+        Callable[  # we will return a function that is a decorator
+            [Callable[FuncIns, FuncR]],  # taking underlying *user* function
+            _ScriptDecoratedFunction[  # and returning an overloaded method that can additionally return Task or Step
+                FuncIns, FuncR
+            ],
+        ],
     ],
 ]:
-    def decorator(
-        real_function: Callable,
-    ) -> Callable[
-        ScriptIns,
-        Callable[[Callable[FuncIns, FuncR]], Union[Callable[FuncIns, FuncR], Callable[ScriptIns, Union[Task, Step]]]],
-    ]:
-        def new_function(
-            *args: ScriptIns.args, **kwargs: ScriptIns.kwargs
-        ) -> Callable[
-            [Callable[FuncIns, FuncR]], Union[Callable[FuncIns, FuncR], Callable[ScriptIns, Union[Task, Step]]]
-        ]:
-            return real_function(*args, **kwargs)
-
-        return new_function
-
-    return decorator
+    """Adds type hints to the decorated function."""
+    return lambda func: func
 
 
-@_take_annotation_from(Script)  # type: ignore
-def script(**script_kwargs):
+@_add_type_hints(Script)
+def script(**script_kwargs) -> Callable:
     """A decorator that wraps a function into a Script object.
 
     Using this decorator users can define a function that will be executed as a script in a container. Once the
@@ -553,10 +683,8 @@ def script(**script_kwargs):
         Function that wraps a given function into a `Script`.
     """
 
-    def script_wrapper(
-        func: Callable[FuncIns, FuncR],
-    ) -> Union[Callable[FuncIns, FuncR], Callable[ScriptIns, Union[Task, Step]]]:
-        """Wraps the given callable into a `Script` object that can be invoked.
+    def script_wrapper(func: Callable[FuncIns, FuncR]) -> Callable:
+        """Wraps the given callable so it can be invoked as a Step or Task.
 
         Parameters
         ----------
@@ -566,8 +694,8 @@ def script(**script_kwargs):
         Returns:
         -------
         Callable
-            Another callable that represents the `Script` object `__call__` method when in a Steps or DAG context,
-            otherwise return the callable function unchanged.
+            Callable that represents the `Script` object `__call__` method when in a Steps or DAG context,
+            otherwise returns the callable function unchanged.
         """
         # instance methods are wrapped in `staticmethod`. Hera can capture that type and extract the underlying
         # function for remote submission since it does not depend on any class or instance attributes, so it is
@@ -587,17 +715,9 @@ def script(**script_kwargs):
 
         s = Script(name=name, source=source, **script_kwargs)
 
-        @overload
-        def task_wrapper(*args: FuncIns.args, **kwargs: FuncIns.kwargs) -> FuncR:
-            ...
-
-        @overload
-        def task_wrapper(*args: ScriptIns.args, **kwargs: ScriptIns.kwargs) -> Union[Step, Task]:
-            ...
-
         @wraps(func)
-        def task_wrapper(*args, **kwargs):
-            """Invokes a `Script` object's `__call__` method using the given `task_params`."""
+        def task_wrapper(*args, **kwargs) -> Union[FuncR, Step, Task, None]:
+            """Invokes a `Script` object's `__call__` method using the given SubNode (Step or Task) args/kwargs."""
             if _context.active:
                 return s.__call__(*args, **kwargs)
             return func(*args, **kwargs)
@@ -621,6 +741,13 @@ class InlineScriptConstructor(ScriptConstructor):
 
     add_cwd_to_sys_path: Optional[bool] = None
 
+    @staticmethod
+    def _roundtrip(source):
+        tree = ast.parse(source)
+        if hasattr(ast, "unparse"):
+            return ast.unparse(tree)
+        return ast.unparse(tree)
+
     def _get_param_script_portion(self, instance: Script) -> str:
         """Constructs and returns a script that loads the parameters of the specified arguments.
 
@@ -681,7 +808,7 @@ class InlineScriptConstructor(ScriptConstructor):
         # in order to have consistent looking functions and getting rid of any comments
         # parsing issues.
         # See https://github.com/argoproj-labs/hera/issues/572
-        content = roundtrip(textwrap.dedent(inspect.getsource(instance.source))).splitlines()
+        content = self._roundtrip(textwrap.dedent(inspect.getsource(instance.source))).splitlines()
         for i, line in enumerate(content):
             if line.startswith("def") or line.startswith("async def"):
                 break
@@ -711,6 +838,17 @@ class RunnerScriptConstructor(ScriptConstructor):
     DEFAULT_HERA_OUTPUTS_DIRECTORY: str = "/tmp/hera-outputs"
     """Used as the default value for when the outputs_directory is not set"""
 
+    pydantic_mode: Optional[Literal[1, 2]] = None
+    """Used for selecting the pydantic version used for BaseModels.
+    Allows for using pydantic.v1 BaseModels with pydantic v2.
+    Defaults to the installed version of Pydantic."""
+
+    @validator("pydantic_mode", always=True)
+    def _pydantic_mode(cls, value: Optional[Literal[1, 2]]) -> Optional[Literal[1, 2]]:
+        if value and value > _PYDANTIC_VERSION:
+            raise ValueError("v2 pydantic mode only available for pydantic>=2")
+        return value
+
     def transform_values(self, cls: Type[Script], values: Any) -> Any:
         """A function that can inspect the Script instance and generate the source field."""
         if not callable(values.get("source")):
@@ -722,7 +860,7 @@ class RunnerScriptConstructor(ScriptConstructor):
             "-m",
             "hera.workflows.runner",
             "-e",
-            f'{values["source"].__module__}:{values["source"].__name__}',
+            f"{values['source'].__module__}:{values['source'].__name__}",
         ]
 
         return values
@@ -735,12 +873,22 @@ class RunnerScriptConstructor(ScriptConstructor):
         self, instance: "Script", script: _ModelScriptTemplate
     ) -> _ModelScriptTemplate:
         """A hook to transform the generated script template."""
-        if global_config.experimental_features["script_annotations"]:
+        script_env = []
+
+        if self.outputs_directory:
+            script_env.append(EnvVar(name="hera__outputs_directory", value=self.outputs_directory))
+        if self.pydantic_mode:
+            script_env.append(EnvVar(name="hera__pydantic_mode", value=str(self.pydantic_mode)))
+        if _flag_enabled(_SCRIPT_PYDANTIC_IO_FLAG):
+            script_env.append(EnvVar(name="hera__script_pydantic_io", value=""))
+
+        if script_env:
             if not script.env:
+                # If user did not set any env vars themselves then we need to initialise the list
                 script.env = []
-            script.env.append(EnvVar(name="hera__script_annotations", value=""))
-            if self.outputs_directory:
-                script.env.append(EnvVar(name="hera__outputs_directory", value=self.outputs_directory))
+
+            script.env.extend(script_env)
+
         return script
 
 
