@@ -3,12 +3,14 @@
 See https://argoproj.github.io/argo-workflows/walk-through/steps/
 for more on Steps.
 """
+
 from typing import Any, List, Optional, Union
 
+from hera.shared._pydantic import PrivateAttr
+from hera.workflows._context import _context
+from hera.workflows._meta_mixins import CallableTemplateMixin, ContextMixin
 from hera.workflows._mixins import (
     ArgumentsMixin,
-    CallableTemplateMixin,
-    ContextMixin,
     IOMixin,
     ItemMixin,
     ParameterMixin,
@@ -16,14 +18,34 @@ from hera.workflows._mixins import (
     TemplateInvocatorSubNodeMixin,
     TemplateMixin,
 )
-from hera.workflows.exceptions import InvalidType
+from hera.workflows.exceptions import InvalidType, NodeNameConflict
 from hera.workflows.models import (
+    ParallelSteps,
     Template as _ModelTemplate,
     WorkflowStep as _ModelWorkflowStep,
 )
 from hera.workflows.protocol import Steppable, Templatable
 
 
+class DummyContext:
+    def __enter__(self):
+        pass
+
+    def __exit__(self, *_):
+        pass
+
+
+def parallel():
+    """Open a parallel context within a steps-decorator function.
+
+    When running locally, the context will be a no-op.
+    """
+    if _context.declaring:
+        return Parallel()
+
+    return DummyContext()
+
+
 class Step(
     TemplateInvocatorSubNodeMixin,
     ArgumentsMixin,
@@ -86,9 +108,14 @@ class Parallel(
 
     sub_steps: List[Union[Step, _ModelWorkflowStep]] = []
 
+    _node_names = PrivateAttr(default_factory=set)
+
     def _add_sub(self, node: Any):
         if not isinstance(node, Step):
             raise InvalidType(type(node))
+        if node.name in self._node_names:
+            raise NodeNameConflict(f"Found multiple Steps named: {node.name}")
+        self._node_names.add(node.name)
         self.sub_steps.append(node)
 
     def _build_step(self) -> List[_ModelWorkflowStep]:
@@ -119,6 +146,8 @@ class Steps(
     * All Step objects initialised within a Parallel context will run in parallel.
     """
 
+    _node_names = PrivateAttr(default_factory=set)
+
     sub_steps: List[
         Union[
             Step,
@@ -129,13 +158,13 @@ class Steps(
         ]
     ] = []
 
-    def _build_steps(self) -> Optional[List[List[_ModelWorkflowStep]]]:
+    def _build_steps(self) -> Optional[List[ParallelSteps]]:
         steps = []
         for workflow_step in self.sub_steps:
             if isinstance(workflow_step, Steppable):
-                steps.append(workflow_step._build_step())
+                steps.append(ParallelSteps(__root__=workflow_step._build_step()))
             elif isinstance(workflow_step, _ModelWorkflowStep):
-                steps.append([workflow_step])
+                steps.append(ParallelSteps(__root__=[workflow_step]))
             elif isinstance(workflow_step, List):
                 substeps = []
                 for s in workflow_step:
@@ -145,7 +174,7 @@ class Steps(
                         substeps.append(s)
                     else:
                         raise InvalidType(type(s))
-                steps.append(substeps)
+                steps.append(ParallelSteps(__root__=substeps))
             else:
                 raise InvalidType(type(workflow_step))
 
@@ -154,7 +183,12 @@ class Steps(
     def _add_sub(self, node: Any):
         if not isinstance(node, (Step, Parallel)):
             raise InvalidType(type(node))
-
+        if isinstance(node, Step):
+            if node.name in self._node_names:
+                raise NodeNameConflict(f"Found multiple Step nodes with name: {node.name}")
+            self._node_names.add(node.name)
+        if isinstance(node, Parallel):
+            node._node_names = self._node_names
         self.sub_steps.append(node)
 
     def parallel(self) -> Parallel:
@@ -176,7 +210,7 @@ class Steps(
             fail_fast=self.fail_fast,
             http=None,
             host_aliases=self.host_aliases,
-            init_containers=self.init_containers,
+            init_containers=self._build_init_containers(),
             inputs=self._build_inputs(),
             memoize=self.memoize,
             metadata=self._build_metadata(),
@@ -204,4 +238,9 @@ class Steps(
         )
 
 
-__all__ = ["Steps", "Step", "Parallel"]
+__all__ = [
+    "Steps",
+    "Step",
+    "Parallel",
+    "parallel",
+]
