@@ -1,10 +1,10 @@
 """Volume module provides all Argo volume types that can be used via Hera."""
+
 import uuid
 from enum import Enum
 from typing import List, Optional, Union, cast
 
-from pydantic import root_validator, validator
-
+from hera.shared._pydantic import root_validator, validator
 from hera.workflows.models import (
     AWSElasticBlockStoreVolumeSource as _ModelAWSElasticBlockStoreVolumeSource,
     AzureDiskVolumeSource as _ModelAzureDiskVolumeSource,
@@ -34,12 +34,12 @@ from hera.workflows.models import (
     ProjectedVolumeSource as _ModelProjectedVolumeSource,
     QuobyteVolumeSource as _ModelQuobyteVolumeSource,
     RBDVolumeSource as _ModelRBDVolumeSource,
-    ResourceRequirements,
     ScaleIOVolumeSource as _ModelScaleIOVolumeSource,
     SecretVolumeSource as _ModelSecretVolumeSource,
     StorageOSVolumeSource as _ModelStorageOSVolumeSource,
     Volume as _ModelVolume,
     VolumeMount as _ModelVolumeMount,
+    VolumeResourceRequirements,
     VsphereVirtualDiskVolumeSource as _ModelVsphereVirtualDiskVolumeSource,
 )
 from hera.workflows.validators import validate_storage_units
@@ -96,6 +96,8 @@ class _BaseVolume(_ModelVolumeMount):
         raise NotImplementedError
 
     def _build_volume_mount(self) -> _ModelVolumeMount:
+        assert self.name
+        assert self.mount_path
         return _ModelVolumeMount(
             name=self.name,
             mount_path=self.mount_path,
@@ -110,6 +112,7 @@ class AWSElasticBlockStoreVolumeVolume(_BaseVolume, _ModelAWSElasticBlockStoreVo
     """Representation of AWS elastic block store volume."""
 
     def _build_volume(self) -> _ModelVolume:
+        assert self.name
         return _ModelVolume(
             name=self.name,
             aws_elastic_block_store=_ModelAWSElasticBlockStoreVolumeSource(
@@ -122,6 +125,7 @@ class AzureDiskVolumeVolume(_BaseVolume, _ModelAzureDiskVolumeSource):
     """Representation of an Azure disk volume."""
 
     def _build_volume(self) -> _ModelVolume:
+        assert self.name
         return _ModelVolume(
             name=self.name,
             azure_disk=_ModelAzureDiskVolumeSource(
@@ -139,6 +143,7 @@ class AzureFileVolumeVolume(_BaseVolume, _ModelAzureFileVolumeSource):
     """Representation of an Azure file that can be mounted as a volume."""
 
     def _build_volume(self) -> _ModelVolume:
+        assert self.name
         return _ModelVolume(
             name=self.name,
             azure_file=_ModelAzureFileVolumeSource(
@@ -151,6 +156,7 @@ class CephFSVolumeVolume(_BaseVolume, _ModelCephFSVolumeSource):
     """Representation of a Ceph file system volume."""
 
     def _build_volume(self) -> _ModelVolume:
+        assert self.name
         return _ModelVolume(
             name=self.name,
             cephfs=_ModelCephFSVolumeSource(
@@ -168,6 +174,7 @@ class CinderVolume(_BaseVolume, _ModelCinderVolumeSource):
     """Representation of a Cinder volume."""
 
     def _build_volume(self) -> _ModelVolume:
+        assert self.name
         return _ModelVolume(
             name=self.name,
             cinder=_ModelCinderVolumeSource(
@@ -183,6 +190,7 @@ class ConfigMapVolume(_BaseVolume, _ModelConfigMapVolumeSource):  # type: ignore
     """Representation of a config map volume."""
 
     def _build_volume(self) -> _ModelVolume:
+        assert self.name
         return _ModelVolume(
             name=self.name,
             config_map=_ModelConfigMapVolumeSource(
@@ -195,6 +203,7 @@ class CSIVolume(_BaseVolume, _ModelCSIVolumeSource):
     """Representation of a container service interface volume."""
 
     def _build_volume(self) -> _ModelVolume:
+        assert self.name
         return _ModelVolume(
             name=self.name,
             csi=_ModelCSIVolumeSource(
@@ -211,6 +220,7 @@ class DownwardAPIVolume(_BaseVolume, _ModelDownwardAPIVolumeSource):
     """Representation of a volume passed via the downward API."""
 
     def _build_volume(self) -> _ModelVolume:
+        assert self.name
         return _ModelVolume(
             name=self.name,
             downward_api=_ModelDownwardAPIVolumeSource(default_mode=self.default_mode, items=self.items),
@@ -221,6 +231,7 @@ class EmptyDirVolume(_BaseVolume, _ModelEmptyDirVolumeSource):
     """Representation of an empty dir volume from K8s."""
 
     def _build_volume(self) -> _ModelVolume:
+        assert self.name
         return _ModelVolume(
             name=self.name, empty_dir=_ModelEmptyDirVolumeSource(medium=self.medium, size_limit=self.size_limit)
         )
@@ -230,6 +241,7 @@ class EphemeralVolume(_BaseVolume, _ModelEphemeralVolumeSource):
     """Representation of a volume that uses ephemeral storage shared with the K8s node a pod is scheduled on."""
 
     def _build_volume(self) -> _ModelVolume:
+        assert self.name
         return _ModelVolume(
             name=self.name, ephemeral=_ModelEphemeralVolumeSource(volume_claim_template=self.volume_claim_template)
         )
@@ -239,6 +251,7 @@ class FCVolume(_BaseVolume, _ModelFCVolumeSource):
     """An FV volume representation."""
 
     def _build_volume(self) -> _ModelVolume:
+        assert self.name
         return _ModelVolume(
             name=self.name,
             fc=_ModelFCVolumeSource(
@@ -255,6 +268,7 @@ class FlexVolume(_BaseVolume, _ModelFlexVolumeSource):
     """A Flex volume representation."""
 
     def _build_volume(self) -> _ModelVolume:
+        assert self.name
         return _ModelVolume(
             name=self.name,
             flex_volume=_ModelFlexVolumeSource(
@@ -271,6 +285,7 @@ class FlockerVolume(_BaseVolume, _ModelFlockerVolumeSource):
     """A Flocker volume representation."""
 
     def _build_volume(self) -> _ModelVolume:
+        assert self.name
         return _ModelVolume(
             name=self.name,
             flocker=_ModelFlockerVolumeSource(dataset_name=self.dataset_name, dataset_uuid=self.dataset_uuid),
@@ -285,6 +300,7 @@ class GCEPersistentDiskVolume(_BaseVolume, _ModelGCEPersistentDiskVolumeSource):
     """
 
     def _build_volume(self) -> _ModelVolume:
+        assert self.name
         return _ModelVolume(
             name=self.name,
             gce_persistent_disk=_ModelGCEPersistentDiskVolumeSource(
@@ -297,6 +313,7 @@ class GitRepoVolume(_BaseVolume, _ModelGitRepoVolumeSource):
     """A representation of a Git repo that can be mounted as a volume."""
 
     def _build_volume(self) -> _ModelVolume:
+        assert self.name
         return _ModelVolume(
             name=self.name,
             git_repo=_ModelGitRepoVolumeSource(
@@ -309,6 +326,7 @@ class GlusterfsVolume(_BaseVolume, _ModelGlusterfsVolumeSource):
     """A representation for a Gluster filesystem volume."""
 
     def _build_volume(self) -> _ModelVolume:
+        assert self.name
         return _ModelVolume(
             name=self.name,
             glusterfs=_ModelGlusterfsVolumeSource(endpoints=self.endpoints, path=self.path, read_only=self.read_only),
@@ -319,6 +337,7 @@ class HostPathVolume(_BaseVolume, _ModelHostPathVolumeSource):
     """Representation for a volume that can be mounted from a host path/node location."""
 
     def _build_volume(self) -> _ModelVolume:
+        assert self.name
         return _ModelVolume(name=self.name, host_path=_ModelHostPathVolumeSource(path=self.path, type=self.type))
 
 
@@ -326,6 +345,7 @@ class ISCSIVolume(_BaseVolume, _ModelISCSIVolumeSource):
     """Representation of ISCSI volume."""
 
     def _build_volume(self) -> _ModelVolume:
+        assert self.name
         return _ModelVolume(
             name=self.name,
             iscsi=_ModelISCSIVolumeSource(
@@ -348,6 +368,7 @@ class NFSVolume(_BaseVolume, _ModelNFSVolumeSource):
     """A network file system volume representation."""
 
     def _build_volume(self) -> _ModelVolume:
+        assert self.name
         return _ModelVolume(
             name=self.name, nfs=_ModelNFSVolumeSource(server=self.server, path=self.path, read_only=self.read_only)
         )
@@ -357,6 +378,7 @@ class PhotonPersistentDiskVolume(_BaseVolume, _ModelPhotonPersistentDiskVolumeSo
     """A Photon Persisten Disk representation."""
 
     def _build_volume(self) -> _ModelVolume:
+        assert self.name
         return _ModelVolume(
             name=self.name,
             photon_persistent_disk=_ModelPhotonPersistentDiskVolumeSource(fs_type=self.fs_type, pd_id=self.pd_id),
@@ -367,6 +389,7 @@ class PortworxVolume(_BaseVolume, _ModelPortworxVolumeSource):
     """`PortworxVolume` represents a Portworx volume to mount to a container."""
 
     def _build_volume(self) -> _ModelVolume:
+        assert self.name
         return _ModelVolume(
             name=self.name,
             portworx_volume=_ModelPortworxVolumeSource(
@@ -379,6 +402,7 @@ class ProjectedVolume(_BaseVolume, _ModelProjectedVolumeSource):
     """`ProjectedVolume` represents a projected volume to mount to a container."""
 
     def _build_volume(self) -> _ModelVolume:
+        assert self.name
         return _ModelVolume(
             name=self.name, projected=_ModelProjectedVolumeSource(default_mode=self.default_mode, sources=self.sources)
         )
@@ -388,6 +412,7 @@ class QuobyteVolume(_BaseVolume, _ModelQuobyteVolumeSource):
     """`QuobyteVolume` represents a Quobyte volume to mount to a container."""
 
     def _build_volume(self) -> _ModelVolume:
+        assert self.name
         return _ModelVolume(
             name=self.name,
             quobyte=_ModelQuobyteVolumeSource(
@@ -405,6 +430,7 @@ class RBDVolume(_BaseVolume, _ModelRBDVolumeSource):
     """An RDB volume representation."""
 
     def _build_volume(self) -> _ModelVolume:
+        assert self.name
         return _ModelVolume(
             name=self.name,
             rbd=_ModelRBDVolumeSource(
@@ -424,6 +450,7 @@ class ScaleIOVolume(_BaseVolume, _ModelScaleIOVolumeSource):
     """`ScaleIOVolume` represents a ScaleIO volume to mount to the container."""
 
     def _build_volume(self) -> _ModelVolume:
+        assert self.name
         return _ModelVolume(
             name=self.name,
             scale_io=_ModelScaleIOVolumeSource(
@@ -445,6 +472,7 @@ class SecretVolume(_BaseVolume, _ModelSecretVolumeSource):
     """`SecretVolume` supports mounting a K8s secret as a container volume."""
 
     def _build_volume(self) -> _ModelVolume:
+        assert self.name
         return _ModelVolume(
             name=self.name,
             secret=_ModelSecretVolumeSource(
@@ -457,6 +485,7 @@ class StorageOSVolume(_BaseVolume, _ModelStorageOSVolumeSource):
     """`StorageOSVolume` represents a Storage OS volume to mount."""
 
     def _build_volume(self) -> _ModelVolume:
+        assert self.name
         return _ModelVolume(
             name=self.name,
             storageos=_ModelStorageOSVolumeSource(
@@ -473,6 +502,7 @@ class VsphereVirtualDiskVolume(_BaseVolume, _ModelVsphereVirtualDiskVolumeSource
     """`VsphereVirtualDiskVolume` represents a vSphere virtual disk volume to mount."""
 
     def _build_volume(self) -> _ModelVolume:
+        assert self.name
         return _ModelVolume(
             name=self.name,
             vsphere_volume=_ModelVsphereVirtualDiskVolumeSource(
@@ -492,6 +522,7 @@ class ExistingVolume(_BaseVolume, _ModelPersistentVolumeClaimVolumeSource):
     """
 
     def _build_volume(self) -> _ModelVolume:
+        assert self.name
         return _ModelVolume(
             name=self.name,
             persistent_volume_claim=_ModelPersistentVolumeClaimVolumeSource(
@@ -509,7 +540,7 @@ class Volume(_BaseVolume, _ModelPersistentVolumeClaimSpec):
     """
 
     size: Optional[str] = None  # type: ignore
-    resources: Optional[ResourceRequirements] = None
+    resources: Optional[VolumeResourceRequirements] = None
     metadata: Optional[ObjectMeta] = None
     access_modes: Optional[List[Union[str, AccessMode]]] = [AccessMode.read_write_once]  # type: ignore
     storage_class_name: Optional[str] = None
@@ -534,7 +565,7 @@ class Volume(_BaseVolume, _ModelPersistentVolumeClaimSpec):
     @root_validator(pre=True)
     def _merge_reqs(cls, values):
         if "size" in values and "resources" in values:
-            resources: ResourceRequirements = values.get("resources")
+            resources: VolumeResourceRequirements = values.get("resources")
             if resources.requests is not None:
                 if "storage" in resources.requests:
                     pass  # take the storage specification in resources
@@ -544,9 +575,9 @@ class Volume(_BaseVolume, _ModelPersistentVolumeClaimSpec):
         elif "resources" not in values:
             assert "size" in values, "at least one of `size` or `resources` must be specified"
             validate_storage_units(cast(str, values.get("size")))
-            values["resources"] = ResourceRequirements(requests={"storage": values.get("size")})
+            values["resources"] = VolumeResourceRequirements(requests={"storage": values.get("size")})
         elif "resources" in values:
-            resources = cast(ResourceRequirements, values.get("resources"))
+            resources = cast(VolumeResourceRequirements, values.get("resources"))
             assert resources.requests is not None, "Resource requests are required"
             storage = resources.requests.get("storage")
             assert storage is not None, "At least one of `size` or `resources.requests.storage` must be specified"
@@ -573,6 +604,7 @@ class Volume(_BaseVolume, _ModelPersistentVolumeClaimSpec):
     def _build_volume(self) -> _ModelVolume:
         claim = self._build_persistent_volume_claim()
         assert claim.metadata is not None, "claim metadata is required"
+        assert self.name
         return _ModelVolume(
             name=self.name,
             persistent_volume_claim=_ModelPersistentVolumeClaimVolumeSource(
