@@ -3,29 +3,23 @@
 See https://argoproj.github.io/argo-workflows/workflow-concepts/#the-workflow
 for more on Workflows.
 """
+
+import logging
 import time
 from pathlib import Path
-from types import ModuleType
-from typing import Any, Dict, List, Optional, Type, Union
+from typing import Annotated, Any, Callable, Dict, List, Optional, Type, TypeVar, Union, get_args
 
-try:
-    from typing import Annotated, get_args  # type: ignore
-except ImportError:
-    from typing_extensions import Annotated, get_args  # type: ignore
-
-
-from pydantic import validator
+from typing_extensions import ParamSpec
 
+from hera import _yaml
 from hera.shared import global_config
-from hera.shared._base_model import BaseModel
+from hera.shared._pydantic import BaseModel, validator
+from hera.workflows._meta_mixins import HookMixin, ModelMapperMixin, TemplateDecoratorFuncsMixin
 from hera.workflows._mixins import (
     ArgumentsMixin,
     ArgumentsT,
-    ContextMixin,
-    HookMixin,
     MetricsMixin,
     MetricsT,
-    ModelMapperMixin,
     VolumeMixin,
     VolumesT,
 )
@@ -65,26 +59,15 @@ from hera.workflows.models import (
 from hera.workflows.parameter import Parameter
 from hera.workflows.protocol import Templatable, TTemplate, TWorkflow, VolumeClaimable
 from hera.workflows.service import WorkflowsService
+from hera.workflows.template_set import TemplateSet
 from hera.workflows.workflow_status import WorkflowStatus
 
-_yaml: Optional[ModuleType] = None
-try:
-    import yaml
-
-    _yaml = yaml
-except ImportError:
-    _yaml = None
-
 ImagePullSecretsT = Optional[Union[LocalObjectReference, List[LocalObjectReference], str, List[str]]]
 
 NAME_LIMIT = 63
 
-# The length of the random suffix used for generate_name
-# length (5) from https://github.com/kubernetes/kubernetes/blob/6195f96e/staging/src/k8s.io/apiserver/pkg/storage/names/generate.go#L45
-_SUFFIX_LEN = 5
-# The max name length comes from https://github.com/kubernetes/kubernetes/blob/6195f96e/staging/src/k8s.io/apiserver/pkg/storage/names/generate.go#L44
-# We want to truncate according to SUFFIX_LEN
-_TRUNCATE_LENGTH = NAME_LIMIT - _SUFFIX_LEN
+T = TypeVar("T")
+P = ParamSpec("P")
 
 
 class _WorkflowModelMapper(ModelMapperMixin.ModelMapper):
@@ -95,11 +78,11 @@ class _WorkflowModelMapper(ModelMapperMixin.ModelMapper):
 
 class Workflow(
     ArgumentsMixin,
-    ContextMixin,
     HookMixin,
     VolumeMixin,
     MetricsMixin,
     ModelMapperMixin,
+    TemplateDecoratorFuncsMixin,
 ):
     """The base Workflow class for Hera.
 
@@ -135,7 +118,7 @@ class Workflow(
                 raise InvalidType(f"{type(template)} is not a valid template type")
 
             if isinstance(template, VolumeClaimable):
-                claims = template._build_persistent_volume_claims()
+                claims = template._build_persistent_volume_claims()  # type: ignore
                 # If there are no claims, continue, nothing to add
                 if not claims:
                     continue
@@ -171,7 +154,6 @@ class Workflow(
 
     # ObjectMeta fields - https://argoproj.github.io/argo-workflows/fields/#objectmeta
     annotations: Annotated[Optional[Dict[str, str]], _WorkflowModelMapper("metadata.annotations")] = None
-    cluster_name: Annotated[Optional[str], _WorkflowModelMapper("metadata.cluster_name")] = None
     creation_timestamp: Annotated[Optional[Time], _WorkflowModelMapper("metadata.creation_timestamp")] = None
     deletion_grace_period_seconds: Annotated[
         Optional[int], _WorkflowModelMapper("metadata.deletion_grace_period_seconds")
@@ -181,14 +163,14 @@ class Workflow(
     generate_name: Annotated[Optional[str], _WorkflowModelMapper("metadata.generate_name")] = None
     generation: Annotated[Optional[int], _WorkflowModelMapper("metadata.generation")] = None
     labels: Annotated[Optional[Dict[str, str]], _WorkflowModelMapper("metadata.labels")] = None
-    managed_fields: Annotated[
-        Optional[List[ManagedFieldsEntry]], _WorkflowModelMapper("metadata.managed_fields")
-    ] = None
+    managed_fields: Annotated[Optional[List[ManagedFieldsEntry]], _WorkflowModelMapper("metadata.managed_fields")] = (
+        None
+    )
     name: Annotated[Optional[str], _WorkflowModelMapper("metadata.name")] = None
     namespace: Annotated[Optional[str], _WorkflowModelMapper("metadata.namespace")] = None
-    owner_references: Annotated[
-        Optional[List[OwnerReference]], _WorkflowModelMapper("metadata.owner_references")
-    ] = None
+    owner_references: Annotated[Optional[List[OwnerReference]], _WorkflowModelMapper("metadata.owner_references")] = (
+        None
+    )
     resource_version: Annotated[Optional[str], _WorkflowModelMapper("metadata.resource_version")] = None
     self_link: Annotated[Optional[str], _WorkflowModelMapper("metadata.self_link")] = None
     uid: Annotated[Optional[str], _WorkflowModelMapper("metadata.uid")] = None
@@ -251,12 +233,12 @@ class Workflow(
     arguments: Annotated[
         ArgumentsT,
         _WorkflowModelMapper("spec.arguments", ArgumentsMixin._build_arguments),
-    ]
+    ] = None
     metrics: Annotated[
         MetricsT,
         _WorkflowModelMapper("spec.metrics", MetricsMixin._build_metrics),
-    ]
-    volumes: Annotated[VolumesT, _WorkflowModelMapper("spec.volumes", VolumeMixin._build_volumes)]
+    ] = None
+    volumes: Annotated[VolumesT, _WorkflowModelMapper("spec.volumes", VolumeMixin._build_volumes)] = None
 
     # Hera-specific fields
     workflows_service: Optional[WorkflowsService] = None
@@ -361,12 +343,44 @@ class Workflow(
 
     def to_yaml(self, *args, **kwargs) -> str:
         """Builds the Workflow as an Argo schema Workflow object and returns it as yaml string."""
-        if not _yaml:
-            raise ImportError("`PyYAML` is not installed. Install `hera[yaml]` to bring in the extra dependency")
-        # Set some default options if not provided by the user
-        kwargs.setdefault("default_flow_style", False)
-        kwargs.setdefault("sort_keys", False)
-        return _yaml.dump(self.to_dict(), *args, **kwargs)
+
+        def human_readable_ordering(kv: tuple) -> int:
+            """Key ordering function for ordering in a more human-readable fashion.
+
+            Ordering is:
+            1. "name" keys always first (if present)
+            2. Primitives (not dicts/lists)
+            3. lists
+            4. dict
+            """
+            k, v = kv
+            if k == "name" and isinstance(v, str):
+                return 0
+            if not isinstance(v, (dict, list)):
+                return 1
+            if isinstance(v, list):
+                return 2
+            return 3
+
+        def order_dict(d: dict) -> dict[str, Any]:
+            """Recursively orders `d` by the custom_ordering function by inserting them into a copy of the dict in order."""
+            d_copy: dict[str, Any] = dict()
+            for k, v in sorted(d.items(), key=lambda x: (human_readable_ordering(x), x[0])):
+                if isinstance(v, dict):
+                    d_copy[k] = order_dict(v)
+                elif isinstance(v, list):
+                    if v and isinstance(v[0], dict):
+                        d_copy[k] = [order_dict(i) if isinstance(i, dict) else i for i in v]
+                    elif v and isinstance(v[0], list):
+                        d_copy[k] = [[order_dict(i) for i in sublist] for sublist in v]
+                    else:
+                        d_copy[k] = v
+                else:
+                    d_copy[k] = v
+            return d_copy
+
+        human_ordered_dict = order_dict(self.to_dict())
+        return _yaml.dump(human_ordered_dict, *args, **kwargs)
 
     def create(self, wait: bool = False, poll_interval: int = 5) -> TWorkflow:
         """Creates the Workflow on the Argo cluster.
@@ -377,13 +391,14 @@ class Workflow(
             If false then the workflow is created asynchronously and the function returns immediately.
             If true then the workflow is created and the function blocks until the workflow is done executing.
         poll_interval: int = 5
-            The interval in seconds to poll the workflow status if wait is true. Ignored when wait is true.
+            The interval in seconds to poll the workflow status if wait is true. Ignored when wait is false.
         """
         assert self.workflows_service, "workflow service not initialized"
         assert self.namespace, "workflow namespace not defined"
 
         wf = self.workflows_service.create_workflow(
-            WorkflowCreateRequest(workflow=self.build()), namespace=self.namespace
+            WorkflowCreateRequest(workflow=self.build()),  # type: ignore
+            namespace=self.namespace,
         )
         # set the workflow name to the name returned by the API, which helps cover the case of users relying on
         # `generate_name=True`
@@ -433,7 +448,8 @@ class Workflow(
         assert self.workflows_service, "workflow service not initialized"
         assert self.namespace, "workflow namespace not defined"
         return self.workflows_service.lint_workflow(
-            WorkflowLintRequest(workflow=self.build()), namespace=self.namespace
+            WorkflowLintRequest(workflow=self.build()),  # type: ignore
+            namespace=self.namespace,
         )
 
     def _add_sub(self, node: Any):
@@ -496,5 +512,24 @@ class Workflow(
         assert self.name is not None, "Cannot fetch a workflow link without a workflow name"
         return self.workflows_service.get_workflow_link(self.name)
 
+    def set_entrypoint(self, func: Callable[P, T]) -> Callable[P, T]:
+        """Decorator function to set entrypoint."""
+        if not hasattr(func, "template_name"):
+            raise SyntaxError("`set_entrypoint` decorator must be above template decorator")
+
+        if self.entrypoint is not None:
+            if self.entrypoint == func.template_name:
+                return func
+
+            logging.warning(f"entrypoint is being reassigned from {self.entrypoint} to {func.template_name}")
+
+        self.entrypoint = func.template_name  # type: ignore
+        return func
+
+    def add_template_set(self, template_set: TemplateSet) -> None:
+        """Add the templates stored in the template_set to this Workflow."""
+        for template in template_set.templates:
+            self._add_sub(template)
+
 
 __all__ = ["Workflow"]
