@@ -3,19 +3,13 @@
 See https://argoproj.github.io/argo-workflows/workflow-templates/
 for more on WorkflowTemplates.
 """
+
 from pathlib import Path
-from typing import Dict, Optional, Type, Union, cast
-
-try:
-    from typing import Annotated  # type: ignore
-except ImportError:
-    from typing_extensions import Annotated  # type: ignore
-
-from pydantic import validator
+from typing import Annotated, Dict, Optional, Type, Union, cast
 
 from hera.exceptions import NotFound
-from hera.shared._base_model import BaseModel
-from hera.workflows._mixins import ModelMapperMixin
+from hera.shared._pydantic import BaseModel, validator
+from hera.workflows._meta_mixins import ModelMapperMixin
 from hera.workflows.models import (
     ObjectMeta,
     WorkflowSpec as _ModelWorkflowSpec,
@@ -26,7 +20,15 @@ from hera.workflows.models import (
     WorkflowTemplateUpdateRequest,
 )
 from hera.workflows.protocol import TWorkflow
-from hera.workflows.workflow import _TRUNCATE_LENGTH, Workflow, _WorkflowModelMapper
+from hera.workflows.workflow import NAME_LIMIT, Workflow, _WorkflowModelMapper
+
+# The length of the random suffix used for generate_name
+# length (5) from https://github.com/kubernetes/kubernetes/blob/6195f96e/staging/src/k8s.io/apiserver/pkg/storage/names/generate.go#L45
+_SUFFIX_LEN = 5
+
+# The max name length comes from https://github.com/kubernetes/kubernetes/blob/6195f96e/staging/src/k8s.io/apiserver/pkg/storage/names/generate.go#L44
+# We want to truncate according to SUFFIX_LEN
+_TRUNCATE_LENGTH = NAME_LIMIT - _SUFFIX_LEN
 
 
 class _WorkflowTemplateModelMapper(_WorkflowModelMapper):
@@ -57,7 +59,8 @@ class WorkflowTemplate(Workflow):
         assert self.workflows_service, "workflow service not initialized"
         assert self.namespace, "workflow namespace not defined"
         return self.workflows_service.create_workflow_template(
-            WorkflowTemplateCreateRequest(template=self.build()), namespace=self.namespace
+            WorkflowTemplateCreateRequest(template=self.build()),  # type: ignore
+            namespace=self.namespace,
         )
 
     def get(self) -> TWorkflow:
@@ -88,7 +91,7 @@ class WorkflowTemplate(Workflow):
             return self.create()
         return self.workflows_service.update_workflow_template(
             self.name,
-            WorkflowTemplateUpdateRequest(template=template),
+            WorkflowTemplateUpdateRequest(template=template),  # type: ignore
             namespace=self.namespace,
         )
 
@@ -97,7 +100,8 @@ class WorkflowTemplate(Workflow):
         assert self.workflows_service, "workflow service not initialized"
         assert self.namespace, "workflow namespace not defined"
         return self.workflows_service.lint_workflow_template(
-            WorkflowTemplateLintRequest(template=self.build()), namespace=self.namespace
+            WorkflowTemplateLintRequest(template=self.build()),  # type: ignore
+            namespace=self.namespace,
         )
 
     def build(self) -> TWorkflow:
@@ -144,6 +148,7 @@ class WorkflowTemplate(Workflow):
     def _get_as_workflow(self, generate_name: Optional[str]) -> Workflow:
         workflow = cast(Workflow, Workflow.from_dict(self.to_dict()))
         workflow.kind = "Workflow"
+        workflow.workflows_service = self.workflows_service  # bind this workflow to the same service
 
         if generate_name is not None:
             workflow.generate_name = generate_name
